import sys
import os
import numpy as np
import math
import time
import SimpleITK as sitk
import matplotlib.pyplot as plt

current_path = os.path.dirname(__file__)
sys.path.append(current_path+'/help_function')
sys.path.append(current_path+'/kwave_function')
from niiCook2 import niiCook


l2n = lambda l: np.array(l)
n2l = lambda n: list(n)
start = time.time()


def compare2simulation(A, B, brain_path):


    makeCook = niiCook()
    makeCook.readSavedFile(brain_path)
    brain = makeCook.array.copy()

    #brain = brain/brain.max()
#     brain[brain<=1800] = 0
#     brain[brain=!1800] = 1

    #brain[brain!=0] = 1
    #brain = np.abs(brain-1)
    
    A = np.multiply(A,  brain)
    B = np.multiply(B,  brain)

    ###################################################
    ## Peak pressure error
    ###################################################

    A_peak = A.max()
    B_peak = B.max()
    #print("A peak:", A_peak, "B peak:", B_peak)
    peak_error = np.abs(A_peak-B_peak)/A_peak

    ###################################################
    ## DSC
    ###################################################

    A[A >= A.max()*0.9] = 1
    A[A < A.max()*0.9] = 0

    B[B >= B.max()*0.9] = 1
    B[B < B.max()*0.9] = 0

    A_volume = np.count_nonzero(A)
    B_volume = np.count_nonzero(B)


    overlab_volume = A*B
    overlab_volume = np.count_nonzero(overlab_volume)

    DSC = (overlab_volume*2)/(A_volume+B_volume)

    ###################################################
    ## Center
    ###################################################

    _, A_center, _, _ = makeCook.makeFWHM(A)
    _, B_center, _, _ = makeCook.makeFWHM(B)

    dist = np.linalg.norm(l2n(B_center)-l2n(A_center))

    return peak_error, DSC, dist

def makefigure_using_arr(real_arr, AI_arr, save_path):

    diff = np.abs(real_arr - AI_arr)

    fig = plt.figure(figsize=(10, 10))

    Rmax = real_arr.max()
    Amax = AI_arr.max()

    if Amax>Rmax:
        vmax = Amax
    else:
        vmax = Rmax

    max_idx = np.squeeze(np.where(real_arr==Rmax))


    ax1 = fig.add_subplot(3, 3, 1)
    ax1.imshow(np.squeeze(real_arr[:, max_idx[1], :]), cmap='jet', interpolation='gaussian',
               vmin=real_arr.min(), vmax=vmax)

    ax2 = fig.add_subplot(3, 3, 2)
    ax2.imshow(np.squeeze(AI_arr[:, max_idx[1], :]), cmap='jet', interpolation='gaussian',
               vmin=real_arr.min(), vmax=vmax)

    ax3 = fig.add_subplot(3, 3, 3)
    ax3.imshow(np.squeeze(diff[:, max_idx[1], :]), cmap='jet', interpolation='gaussian',
               vmin=real_arr.min(), vmax=vmax)

    ax1 = fig.add_subplot(3, 3, 4)
    ax1.imshow(np.squeeze(real_arr[max_idx[0], :, :]), cmap='jet', interpolation='gaussian',
               vmin=real_arr.min(), vmax=vmax)

    ax2 = fig.add_subplot(3, 3, 5)
    ax2.imshow(np.squeeze(AI_arr[max_idx[0], :, :]), cmap='jet', interpolation='gaussian',
               vmin=real_arr.min(), vmax=vmax)

    ax3 = fig.add_subplot(3, 3, 6)
    ax3.imshow(np.squeeze(diff[max_idx[0], :, :]), cmap='jet', interpolation='gaussian',
               vmin=real_arr.min(), vmax=vmax)

    ax1 = fig.add_subplot(3, 3, 7)
    ax1.imshow(np.squeeze(real_arr[:, :, max_idx[2]]), cmap='jet', interpolation='gaussian',
               vmin=real_arr.min(), vmax=vmax)

    ax2 = fig.add_subplot(3, 3, 8)
    ax2.imshow(np.squeeze(AI_arr[:, :, max_idx[2]]), cmap='jet', interpolation='gaussian',
               vmin=real_arr.min(), vmax=vmax)

    ax3 = fig.add_subplot(3, 3, 9)
    ax3.imshow(np.squeeze(diff[:, :, max_idx[2]]), cmap='jet', interpolation='gaussian',
               vmin=real_arr.min(), vmax=vmax)

    plt.savefig(save_path, dpi=300)