import sys
import os
import numpy as np
import matplotlib.pyplot as plt
import math
import time
import scipy.io as sio
import SimpleITK as sitk
import os
import glob
import cv2
from  scipy import ndimage

def load_ARC(target, path, ARC_num):
    file_path = os.path.join(path, target + '*')
    file_name = glob.glob(file_path)[0]
    ARC_list = np.loadtxt(file_name)[:ARC_num, 4:]
    ARC_list = np.round(ARC_list, 3)

    brain_pos = file_name.split("[")[1].split("]")[0].split(" ")
    target_pos = np.array([float(brain_pos[0]), float(brain_pos[1]), float(brain_pos[2])])

    return target_pos, ARC_list

def segmentation_mask(arr, threshold):

    arr = np.array(arr * 3000, dtype='float32')
    ret, img_th = cv2.threshold(arr, threshold, 1, 0)

    # remain largest region
    label_im, nb_labels = ndimage.label(img_th)
    sizes = ndimage.sum(img_th, label_im, range(nb_labels + 1))
    mask = sizes == np.max(sizes)
    label_img = mask[label_im]

    # seg_arr = np.multiply(arr, label_img)

    if threshold == -100:
        kernel = np.ones((3, 3), np.uint8)

        open_volume = cv2.morphologyEx(np.array(label_img).astype(np.uint8), cv2.MORPH_OPEN, kernel)

        label_im, nb_labels = ndimage.label(open_volume)
        sizes = ndimage.sum(img_th, label_im, range(nb_labels + 1))
        mask = sizes == np.max(sizes)
        label_img = mask[label_im]
        open_volume = np.array(label_img).astype(np.uint8)

        closed_volume = cv2.morphologyEx(open_volume, cv2.MORPH_CLOSE, kernel)
        # closed_volume = ndimage.binary_erosion(closed_volume)
        #         temp =  closed_volume
        temp = np.zeros_like(label_img)

        for idx in range(closed_volume.shape[0]):
            closed_slice = closed_volume[idx, :, :]
            mask_slice = ndimage.morphology.binary_fill_holes(closed_slice)
            temp[idx, :, :] = mask_slice
        #   mpimg.imsave(os.path.join('E:/2020/KHM_synt/seg', str(threshold)+'_'+str(idx)+'.png'), mask_slice, vmin=-1, vmax=1, cmap='gray')
        temp = ndimage.binary_erosion(temp)

    if threshold == 220:
        kernel = np.ones((3, 3), np.uint8)
        open_volume = cv2.morphologyEx(np.array(label_img).astype(np.uint8), cv2.MORPH_OPEN, kernel)

        label_im, nb_labels = ndimage.label(open_volume)
        sizes = ndimage.sum(img_th, label_im, range(nb_labels + 1))
        mask = sizes == np.max(sizes)
        label_img = mask[label_im]
        open_volume = np.array(label_img).astype(np.uint8)

        closed_volume = cv2.morphologyEx(open_volume, cv2.MORPH_CLOSE, kernel)
        temp = closed_volume

    skull_volume = np.array(temp)

    # extract brain region only
    mask = np.copy(skull_volume)

    mask[:10, :, :] = 1
    mask[-10:, :, :] = 1
    mask[:, :, :10] = 1
    mask[:, :, -10:] = 1
    mask = mask - 1
    mask = -mask
    brain_seg = np.zeros_like(mask)

    for i in range(140):
        ret, makers = cv2.connectedComponents(mask[:, i, :])
        brain_seg[:, i, :] = makers

    brain_seg[brain_seg <= 1] = 0
    brain_seg[brain_seg > 0] = 1

    #return skull_volume, brain_seg
    return brain_seg


def DSC_FWHM(y_true, y_pred, brain_mask):

    y_true = np.multiply(y_true, brain_mask)
    y_pred = np.multiply(y_pred, brain_mask)

    y_true[y_true >= np.max(y_true) * 0.5] = 1
    y_true[y_true < np.max(y_true) * 0.5] = 0
    y_pred[y_pred >= np.max(y_pred) * 0.5] = 1
    y_pred[y_pred < np.max(y_pred) * 0.5] = 0


    overlab = np.count_nonzero(np.multiply(y_pred, y_true))

    true_FWHM = np.count_nonzero(y_true)
    pred_FWHM = np.count_nonzero(y_pred)

    DSC = overlab * 2 / (true_FWHM + pred_FWHM)
    #loss = 1 - DSC

    return DSC

