import numpy as np
import SimpleITK as sitk
import scipy.io as sio
from tqdm import tqdm
from numba import jit
from numba import njit, prange, cuda

l2n = lambda l: np.array(l)
n2l = lambda n: list(n)

class class_calculateScore():

    def __init__(self):
        dummy = 0

    def get_data(self, package):

        self.ROI_idx  = package[0]
        self.back_source = package[1]
        self.restart = package[1]
        self.PHASE= package[2]
        self.AMP= package[3]
        self.skullCrop_arr= package[4]
        self.width= package[5]
        self.ROC= package[6]
        self.dx= package[7]
        self.gather_score= package[8]
        self.skullCrop_itk= package[9]
        self.gather_point= package[10]

    def calculateScore(self, Input_data):

        Input_data = l2n(Input_data)

        TCenter_normalized = Input_data[:3]
        Tnormal = Input_data[3:]
        Tnormal = Tnormal / np.linalg.norm(Tnormal)

        ROI_idx = self.ROI_idx
        TCenter = TCenter_normalized.copy()

        x_range = l2n([ROI_idx[:, 0].min(), ROI_idx[:, 0].max()])
        y_range = l2n([ROI_idx[:, 1].min(), ROI_idx[:, 1].max()])
        z_range = l2n([ROI_idx[:, 2].min(), ROI_idx[:, 2].max()])

        TCenter[0] = TCenter_normalized[0] * (x_range.max() - x_range.min()) + x_range.min()
        TCenter[1] = TCenter_normalized[1] * (y_range.max() - y_range.min()) + y_range.min()
        TCenter[2] = TCenter_normalized[2] * (z_range.max() - z_range.min()) + z_range.min()

        TCenter = TCenter.astype(int)

        ## Check Normal vector

        angle = []
        for i in range(len(self.back_source)):
            back_target_idx = self.back_source[i]
            standard_vector = l2n(back_target_idx) - l2n(TCenter)

            standard_vector = standard_vector / np.linalg.norm(standard_vector)
            dot_product = np.dot(Tnormal, standard_vector)
            angle.append(np.abs(np.rad2deg(np.arccos(dot_product))))

        if angle[0] > 20:
            score = 0
            self.restart = self.restart + 1
            # print("Invalide",self.restart, score)
            return score

        score, Spos = fast_score_multi_source(TCenter, Tnormal, self.PHASE, self.AMP, self.skullCrop_arr,
                                                  self.width, self.ROC, self.dx)
        self.gather_score.append(score)
        self.restart = self.restart + 1

        if score != 0:

            if self.optimizer_check == 0:
                print("Optimizer enter the orbit")
                self.optimizer_check = 1

            point = self.skullCrop_itk.TransformIndexToPhysicalPoint(
                (int(TCenter[0]), int(TCenter[1]), int(TCenter[2])))
            point = l2n(point) * l2n([-1, -1, 1])

            final_data = np.zeros(7)
            final_data[:3] = point
            final_data[3:6] = Tnormal
            final_data[-1] = score

            self.gather_point.append(final_data)
        # print("valide",self.restart, -score)
        # print("pos:",TCenter, "norm:",Tnormal, "Score:",score)

        return -score




@jit(nopython=True, parallel=True, fastmath= True)
def make_fast_ROI(ROI_idx, p_raw, times, BP_Phase, BP_Amp, BP_step,period):
    for i in range(ROI_idx.shape[0]):
        record = p_raw[ROI_idx[i, 0], ROI_idx[i, 1], ROI_idx[i, 2], :]
        peaks = np.argmax(record)
        TOF = times[peaks]
        Amp = record[peaks]
        BP_Phase[ROI_idx[i, 0], ROI_idx[i, 1], ROI_idx[i, 2]] = 2 * np.pi * ((TOF / period) - np.floor(TOF / period))
        BP_Amp[ROI_idx[i, 0], ROI_idx[i, 1], ROI_idx[i, 2]] = Amp
        BP_step[ROI_idx[i, 0], ROI_idx[i, 1], ROI_idx[i, 2]] = np.floor(TOF / period)

    return BP_Phase, BP_Amp, BP_step


@jit(nopython=True)
def make_transducer(ROC, width, dx, Tcenter, Tnormal):

    Tdia = width / 1000
    Troc = ROC / 1000
    pi = 3.141592653589793

    H = Troc - np.sqrt(Troc ** 2 - (0.5 * Tdia) ** 2)
    TP_dis = np.floor(H / dx)
    move_Tcenter = (Tcenter + TP_dis * Tnormal)

    nH = 2000
    nD = 2000
    nA = int(np.floor(0.5 * Tdia / dx) + 5)
    A = np.zeros((2 * nA + 1, 2 * nA + 1, 2 * nA + 1))

    Vs = np.zeros(3)
    Vs[1] = -Tnormal[2]
    Vs[2] = Tnormal[1]

    if np.all(Vs == 0):
        Vs[1] = 0.000000000001
        Vs[2] = 0.000000000001

    Vs = Vs / np.linalg.norm(Vs)
    Vt = np.cross(Tnormal, Vs)

    for i in range(nH):
        R = np.sqrt(Troc ** 2 - (Troc - H + i * H / nH) ** 2)
        for j in range(nD):
            theta = 2 * pi / nD * j
            X = int((nA + 1) + np.round(
                (-(i) * H / nH * Tnormal[0] + R * np.cos(theta) * Vs[0] + R * np.sin(theta) * Vt[0]) / dx))
            Y = int((nA + 1) + np.round(
                (-(i) * H / nH * Tnormal[1] + R * np.cos(theta) * Vs[1] + R * np.sin(theta) * Vt[1]) / dx))
            Z = int((nA + 1) + np.round(
                (-(i) * H / nH * Tnormal[2] + R * np.cos(theta) * Vs[2] + R * np.sin(theta) * Vt[2]) / dx))
            A[X, Y, Z] = 1

    nS = int(np.sum(A))
    Spos = np.zeros((nS, 4))
    idx = np.where(A == 1)

    Spos[:, 2] = idx[0] - (nA + 1) + move_Tcenter[0]
    Spos[:, 1] = idx[1] - (nA + 1) + move_Tcenter[1]
    Spos[:, 0] = idx[2] - (nA + 1) + move_Tcenter[2]

    return Spos

@njit(parallel=True, fastmath= True)
def fast_score(ROI_idx, BP_Phase, BP_Amp, skullCrop_arr, width, ROC, dx):

    # ROI_idx = np.zeros((skullCrop_arr.shape[1],3))
    # ROI_idx[:,0] = 138
    # ROI_idx[:, 1] = np.arange(skullCrop_arr.shape[1])
    # ROI_idx[:, 2] = 138

    Tnormal = np.zeros(3)
    Tnormal[0] = 0
    Tnormal[1] = 1
    Tnormal[0] = 0

    gather_score = np.zeros(ROI_idx.shape[0])
    gather_score[:] = -10

    half_width = (width/2)/(dx*1000)
    Tran_extent = np.zeros((4,3))
    # 1256684
    for tran_idx in prange(int(ROI_idx.shape[0])):
    #for tran_idx in range(ROI_idx.shape[0] - 10000, ROI_idx.shape[0]):#prange(ROI_idx.shape[0]):

        if tran_idx%1000 == 0:
            print("Ing...", 100*(tran_idx/ROI_idx.shape[0]))
            #print("find point 1", tran_idx)

        Tcenter = ROI_idx[tran_idx, :]

        # Tran_extent[0,:] = Tcenter
        # Tran_extent[0, 0] = Tran_extent[0, 0] - half_width
        #
        # Tran_extent[1,:] = Tcenter
        # Tran_extent[1, 0] = Tran_extent[0, 0] + half_width
        #
        # Tran_extent[2, :] = Tcenter
        # Tran_extent[2, 2] = Tran_extent[0, 2] - half_width
        #
        # Tran_extent[3, :] = Tcenter
        # Tran_extent[3, 2] = Tran_extent[0, 2] + half_width
        #
        # if np.any(Tran_extent[:,0]>skullCrop_arr.shape[0])\
        #         or np.any(Tran_extent[:,2]>skullCrop_arr.shape[2])\
        #         or np.any(Tran_extent<0):
        #     continue

        ################################################################################################
        #### make transducer
        ################################################################################################

        Tdia = width / 1000
        Troc = ROC / 1000
        pi = 3.141592653589793

        H = Troc - np.sqrt(Troc ** 2 - (0.5 * Tdia) ** 2)
        TP_dis = np.floor(H / dx)
        move_Tcenter = (Tcenter + TP_dis * Tnormal)

        nH = 1000
        nD = 1000
        nA = int(np.floor(0.5 * Tdia / dx) + 5)
        A = np.zeros((2 * nA + 1, 2 * nA + 1, 2 * nA + 1))

        Vs = np.zeros(3)
        Vs[1] = -Tnormal[2]
        Vs[2] = Tnormal[1]

        Vs = Vs / np.linalg.norm(Vs)
        Vt = np.cross(Tnormal, Vs)
        #print("find point 2", tran_idx)
        for i in range(nH):
            R = np.sqrt(Troc ** 2 - (Troc - H + i * H / nH) ** 2)
            for j in range(nD):
                theta = 2 * pi / nD * j
                X = int((nA + 1) + np.round(
                    (-(i) * H / nH * Tnormal[0] + R * np.cos(theta) * Vs[0] + R * np.sin(theta) * Vt[0]) / dx))
                Y = int((nA + 1) + np.round(
                    (-(i) * H / nH * Tnormal[1] + R * np.cos(theta) * Vs[1] + R * np.sin(theta) * Vt[1]) / dx))
                Z = int((nA + 1) + np.round(
                    (-(i) * H / nH * Tnormal[2] + R * np.cos(theta) * Vs[2] + R * np.sin(theta) * Vt[2]) / dx))
                A[X, Y, Z] = 1

        nS = int(np.sum(A))
        Spos = np.zeros((nS, 4))
        idx = np.where(A == 1)

        Spos[:, 2] = (idx[0] - (nA + 1) + move_Tcenter[0])
        Spos[:, 1] = (idx[1] - (nA + 1) + move_Tcenter[1])
        Spos[:, 0] = (idx[2] - (nA + 1) + move_Tcenter[2])

        ################################################################################################
        #### Check ROI or not
        ################################################################################################

        if np.any(Spos[:, 0] >= skullCrop_arr.shape[0]) \
                or np.any(Spos[:, 1] >= skullCrop_arr.shape[1]) \
                or np.any(Spos[:, 2] >= skullCrop_arr.shape[2])\
                or np.any(Spos < 0):

            continue

        ################################################################################################
        #### Calculate score
        ################################################################################################
        check = np.zeros(Spos.shape[0])
        phase = np.zeros(Spos.shape[0])
        amp = np.zeros(Spos.shape[0])

        BP_Amp = (BP_Amp/BP_Amp.max())*1000
        #print("find point 3", tran_idx)

        for p in range(Spos.shape[0]):
            check[p] = skullCrop_arr[int(Spos[p, 0]), int(Spos[p, 1]), int(Spos[p, 2])]
            phase[p] = BP_Phase[int(Spos[p, 0]), int(Spos[p, 1]), int(Spos[p, 2])]
            amp[p] = BP_Amp[int(Spos[p, 0]), int(Spos[p, 1]), int(Spos[p, 2])]

            # check transducer out of ROI
        #print("find point 4", tran_idx)

        if np.any(amp == 0) or np.any(check > 250):
            continue


        temp = np.zeros((Spos.shape[0], 2))
        temp[:,0] = amp * np.cos(phase)
        temp[:,1] = amp * np.sin(phase)

        temp_sum = np.sum(temp, axis=0)
        #print("find point 4-1", tran_idx)

        score = np.abs(np.complex(temp_sum[0], temp_sum[1]))

       # print("find point 4-2", tran_idx)

        if tran_idx % 1000 == 0:
            print("Ing...", 100 * (tran_idx / ROI_idx.shape[0]))
            print(tran_idx)

        #print("find point 5", tran_idx)
        gather_score[tran_idx] = score

    return gather_score

@jit(nopython=True)
def fast_score2(Tcenter, BP_Phase, BP_Amp, skullCrop_arr, width, ROC, dx):

    Tnormal = np.zeros(3)
    Tnormal[0] = 0
    Tnormal[1] = 1
    Tnormal[0] = 0

    ################################################################################################
    #### make transducer
    ################################################################################################

    Tdia = width / 1000
    Troc = ROC / 1000
    pi = 3.141592653589793

    H = Troc - np.sqrt(Troc ** 2 - (0.5 * Tdia) ** 2)
    TP_dis = np.floor(H / dx)
    move_Tcenter = (Tcenter + TP_dis * Tnormal)

    nH = 1000
    nD = 1000
    nA = int(np.floor(0.5 * Tdia / dx) + 5)
    A = np.zeros((2 * nA + 1, 2 * nA + 1, 2 * nA + 1))

    Vs = np.zeros(3)
    Vs[1] = -Tnormal[2]
    Vs[2] = Tnormal[1]

    Vs = Vs / np.linalg.norm(Vs)
    Vt = np.cross(Tnormal, Vs)
    #print("find point 2", tran_idx)
    for i in range(nH):
        R = np.sqrt(Troc ** 2 - (Troc - H + i * H / nH) ** 2)
        for j in range(nD):
            theta = 2 * pi / nD * j
            X = int((nA + 1) + np.round(
                (-(i) * H / nH * Tnormal[0] + R * np.cos(theta) * Vs[0] + R * np.sin(theta) * Vt[0]) / dx))
            Y = int((nA + 1) + np.round(
                (-(i) * H / nH * Tnormal[1] + R * np.cos(theta) * Vs[1] + R * np.sin(theta) * Vt[1]) / dx))
            Z = int((nA + 1) + np.round(
                (-(i) * H / nH * Tnormal[2] + R * np.cos(theta) * Vs[2] + R * np.sin(theta) * Vt[2]) / dx))
            A[X, Y, Z] = 1

    nS = int(np.sum(A))
    Spos = np.zeros((int(nS), 4))
    idx = np.where(A == 1)

    Spos[:, 2] = (idx[0] - (nA + 1) + move_Tcenter[0])
    Spos[:, 1] = (idx[1] - (nA + 1) + move_Tcenter[1])
    Spos[:, 0] = (idx[2] - (nA + 1) + move_Tcenter[2])

    ################################################################################################
    #### Check ROI or not
    ################################################################################################

    if np.any(Spos[:, 0] >= skullCrop_arr.shape[0])\
            or np.any(Spos[:, 1] >= skullCrop_arr.shape[1])\
            or np.any(Spos[:, 2] >= skullCrop_arr.shape[2])\
            or np.any(Spos < 0):

        score = 0
        return score, Spos

    ################################################################################################
    #### Calculate score
    ################################################################################################
    check = np.zeros(Spos.shape[0])
    phase = np.zeros(Spos.shape[0])
    amp = np.zeros(Spos.shape[0])

    BP_Amp = (BP_Amp/BP_Amp.max())

    for p in range(Spos.shape[0]):
        check[p] = skullCrop_arr[int(Spos[p, 0]), int(Spos[p, 1]), int(Spos[p, 2])]
        phase[p] = BP_Phase[int(Spos[p, 0]), int(Spos[p, 1]), int(Spos[p, 2])]
        amp[p] = BP_Amp[int(Spos[p, 0]), int(Spos[p, 1]), int(Spos[p, 2])]


    if np.any(amp == 0) or np.any(check > 250):
        score = 0
        return score, Spos

    temp = np.zeros((Spos.shape[0], 2))
    temp[:,0] = np.multiply(amp,  np.cos(phase))
    temp[:,1] = np.multiply(amp,  np.sin(phase))
    temp_sum = np.sum(temp, axis=0)
    score = np.abs(np.complex(temp_sum[0], temp_sum[1]))

    return score, Spos

@jit(nopython=True)
def fast_score3(Tcenter, Tnormal,  BP_Phase, BP_Amp, skullCrop_arr, width, ROC, dx):

    ################################################################################################
    #### make transducer
    ################################################################################################

    Tdia = width / 1000
    Troc = ROC / 1000
    pi = 3.141592653589793

    H = Troc - np.sqrt(Troc ** 2 - (0.5 * Tdia) ** 2)
    TP_dis = np.floor(H / dx)
    move_Tcenter = (Tcenter + TP_dis * Tnormal)

    nH = 1000
    nD = 1000
    nA = int(np.floor(0.5 * Tdia / dx) + 5)
    A = np.zeros((2 * nA + 1, 2 * nA + 1, 2 * nA + 1))

    Vs = np.zeros(3)
    Vs[1] = -Tnormal[2]
    Vs[2] = Tnormal[1]

    Vs = Vs / np.linalg.norm(Vs)
    Vt = np.cross(Tnormal, Vs)
    #print("find point 2", tran_idx)
    for i in range(nH):
        R = np.sqrt(Troc ** 2 - (Troc - H + i * H / nH) ** 2)
        for j in range(nD):
            theta = 2 * pi / nD * j
            X = int((nA + 1) + np.round(
                (-(i) * H / nH * Tnormal[0] + R * np.cos(theta) * Vs[0] + R * np.sin(theta) * Vt[0]) / dx))
            Y = int((nA + 1) + np.round(
                (-(i) * H / nH * Tnormal[1] + R * np.cos(theta) * Vs[1] + R * np.sin(theta) * Vt[1]) / dx))
            Z = int((nA + 1) + np.round(
                (-(i) * H / nH * Tnormal[2] + R * np.cos(theta) * Vs[2] + R * np.sin(theta) * Vt[2]) / dx))
            A[X, Y, Z] = 1

    nS = int(np.sum(A))
    Spos = np.zeros((int(nS), 4))
    idx = np.where(A == 1)

    Spos[:, 2] = (idx[0] - (nA + 1) + move_Tcenter[0])
    Spos[:, 1] = (idx[1] - (nA + 1) + move_Tcenter[1])
    Spos[:, 0] = (idx[2] - (nA + 1) + move_Tcenter[2])

    ################################################################################################
    #### Check ROI or not
    ################################################################################################

    if np.any(Spos[:, 0] >= skullCrop_arr.shape[0])\
            or np.any(Spos[:, 1] >= skullCrop_arr.shape[1])\
            or np.any(Spos[:, 2] >= skullCrop_arr.shape[2])\
            or np.any(Spos < 0):

        score = 0
        return score, Spos

    ################################################################################################
    #### Calculate score
    ################################################################################################
    check = np.zeros(Spos.shape[0])
    phase = np.zeros(Spos.shape[0])
    amp = np.zeros(Spos.shape[0])

    BP_Amp = (BP_Amp/BP_Amp.max())

    for p in range(Spos.shape[0]):
        check[p] = skullCrop_arr[int(Spos[p, 0]), int(Spos[p, 1]), int(Spos[p, 2])]
        phase[p] = BP_Phase[int(Spos[p, 0]), int(Spos[p, 1]), int(Spos[p, 2])]
        amp[p] = BP_Amp[int(Spos[p, 0]), int(Spos[p, 1]), int(Spos[p, 2])]


    if np.any(amp == 0) or np.any(check > 250):
        score = 0
        return score, Spos

    temp = np.zeros((Spos.shape[0], 2))
    temp[:,0] = np.multiply(amp,  np.cos(phase))
    temp[:,1] = np.multiply(amp,  np.sin(phase))
    temp_sum = np.sum(temp, axis=0)
    score = np.abs(np.complex(temp_sum[0], temp_sum[1]))

    return score, Spos

@jit(nopython=True)
def fast_score_multi_source(Tcenter, Tnormal,  PHASE, AMP, skullCrop_arr, width, ROC, dx):

    ################################################################################################
    #### make transducer
    ################################################################################################

    Tdia = width / 1000
    Troc = ROC / 1000
    pi = 3.141592653589793

    H = Troc - np.sqrt(Troc ** 2 - (0.5 * Tdia) ** 2)
    TP_dis = np.floor(H / dx)
    move_Tcenter = (Tcenter + TP_dis * Tnormal)

    nH = 1500
    nD = 1500
    nA = int(np.floor(0.5 * Tdia / dx) + 5)
    A = np.zeros((2 * nA + 1, 2 * nA + 1, 2 * nA + 1))

    Vs = np.zeros(3)
    Vs[1] = -Tnormal[2]
    Vs[2] = Tnormal[1]

    Vs = Vs / np.linalg.norm(Vs)
    Vt = np.cross(Tnormal, Vs)
    #print("find point 2", tran_idx)
    for i in range(nH):
        R = np.sqrt(Troc ** 2 - (Troc - H + i * H / nH) ** 2)
        for j in range(nD):
            theta = 2 * pi / nD * j
            X = int((nA + 1) + np.round(
                (-(i) * H / nH * Tnormal[0] + R * np.cos(theta) * Vs[0] + R * np.sin(theta) * Vt[0]) / dx))
            Y = int((nA + 1) + np.round(
                (-(i) * H / nH * Tnormal[1] + R * np.cos(theta) * Vs[1] + R * np.sin(theta) * Vt[1]) / dx))
            Z = int((nA + 1) + np.round(
                (-(i) * H / nH * Tnormal[2] + R * np.cos(theta) * Vs[2] + R * np.sin(theta) * Vt[2]) / dx))
            A[X, Y, Z] = 1

    nS = int(np.sum(A))
    Spos = np.zeros((int(nS), 4))
    idx = np.where(A == 1)

    Spos[:, 2] = (idx[0] - (nA + 1) + move_Tcenter[0])
    Spos[:, 1] = (idx[1] - (nA + 1) + move_Tcenter[1])
    Spos[:, 0] = (idx[2] - (nA + 1) + move_Tcenter[2])

    ################################################################################################
    #### Check ROI or not
    ################################################################################################

    if np.any(Spos[:, 0] >= skullCrop_arr.shape[0])\
            or np.any(Spos[:, 1] >= skullCrop_arr.shape[1])\
            or np.any(Spos[:, 2] >= skullCrop_arr.shape[2])\
            or np.any(Spos < 0):

        score = 0
        return score, Spos

    ################################################################################################
    #### Calculate score
    ################################################################################################
    check = np.zeros(Spos.shape[0])
    phase = np.zeros(Spos.shape[0])
    amp = np.zeros(Spos.shape[0])

    gather_score = []
    for i in range(len(PHASE)):

        BP_Phase = PHASE[i]
        BP_Amp = AMP[i]

        for p in range(Spos.shape[0]):
            check[p] = skullCrop_arr[int(Spos[p, 0]), int(Spos[p, 1]), int(Spos[p, 2])]
            phase[p] = BP_Phase[int(Spos[p, 0]), int(Spos[p, 1]), int(Spos[p, 2])]
            amp[p] = BP_Amp[int(Spos[p, 0]), int(Spos[p, 1]), int(Spos[p, 2])]

        if np.any(amp == 0) or np.any(check > 250):
            score = 0
            return score, Spos

        temp = np.zeros((Spos.shape[0], 2))
        temp[:,0] = np.multiply(amp,  np.cos(phase))
        temp[:,1] = np.multiply(amp,  np.sin(phase))
        temp_sum = np.sum(temp, axis=0)

        score_at1 = np.abs(np.complex(temp_sum[0], temp_sum[1]))
        gather_score.append(score_at1)

    score = np.sum(np.array(gather_score))

    return score, Spos

def cordi2idx(image_array, point):

    idx = np.zeros((3))
    for i in range(3):
        idx[i] = np.abs(image_array[i] - point[i]).argmin()

    return idx

def neper2db(alpha, y):

    alphaDB = 20*np.log10(np.exp(1))*alpha*pow((2*np.pi*1e6), y)/100

    return alphaDB

def saveITK(path, itkImage):

    writer = sitk.ImageFileWriter()
    writer.SetFileName(path)
    writer.Execute(itkImage)