import SimpleITK as sitk
import numpy as np
import scipy.io as sio
import math

l2n = lambda l: np.array(l)
n2l = lambda n: list(n)

# filePath = "D:\MR&CT data\KHM\KHM_registered_slicer\CT_register"
#
# grid_number  = (108, 108, 236)
# grid_spacing = (0.741, 0.741, 0.741)
#
# transducer  = (40.83677673339843750000, -32.15667343139648437500, 108.49995422363281250000)
# target = (22.7292, -27.1715, 56.572)
#
# transducer  = np.multiply(transducer,  (-1, -1, 1))
# target = np.multiply(target,  (-1, -1, 1))



def DICOM_read_rotation(filePath, width,  grid_spacing, tran_pos, target_pos):

    if filePath[-2:] =="gz" :
        reader = sitk.ImageFileReader()
        reader.SetImageIO("NiftiImageIO")
        reader.SetFileName(filePath)
        image = reader.Execute()
    elif filePath[-3:] == "nii":
        reader = sitk.ImageFileReader()
        reader.SetImageIO("NiftiImageIO")
        reader.SetFileName(filePath)
        image = reader.Execute()
    else:
        reader = sitk.ImageSeriesReader()
        dicom_names = reader.GetGDCMSeriesFileNames(filePath)
        reader.SetFileNames(dicom_names)
        image = reader.Execute()

    focus = np.linalg.norm(tran_pos - target_pos)

    Nx = np.power(2, (math.ceil(math.log2(width / grid_spacing[0] + 20))))
    Ny = Nx
    Nz = np.power(2, (math.ceil(math.log2(2 * focus / grid_spacing[2] + 20))))

    grid_number = [Nx, Ny, Nz]

    y_end = -grid_spacing[1]*grid_number[1]/2
    x_end = -grid_spacing[0]*grid_number[0]/2
    z_end = -(focus+20*grid_spacing[2])

    grid_origin = (x_end, y_end, z_end)

    reference_image = sitk.Image(int(grid_number[0]), int(grid_number[1]), int(grid_number[2]), sitk.sitkFloat32)
    reference_image.SetSpacing(grid_spacing)
    reference_image.SetOrigin(grid_origin)
    reference_image[:,:,:] = 0
    return image, reference_image


def make_transform(trans_pos, target_pos):

    dir = trans_pos-target_pos

    n_z = np.sqrt(dir[0]*dir[0] + dir[1]*dir[1])
    rotate_z = l2n((( dir[0]/n_z,   dir[1]/n_z,   0),
                    (-dir[1]/n_z,   dir[0]/n_z,   0),
                    (          0,            0,   1)))

    dir2 = np.dot(rotate_z, np.transpose(dir))
    n_y = np.sqrt(dir2[0]*dir2[0] + dir2[2]*dir2[2])
    rotate_y = l2n((( -dir2[2]/n_y,    0,    dir2[0]/n_y),
                    (            0,    1,              0),
                    ( -dir2[0]/n_y,    0,   -dir2[2]/n_y)))

    transform_matix = np.dot(rotate_y, rotate_z)
    transform_matix = np.linalg.inv(transform_matix)
    return transform_matix


def resample(target_pos, image, reference_image, transform_martix):

    move2target = sitk.TranslationTransform(3, target_pos)  # (dimension, move location)
    rigid_euler = sitk.Euler3DTransform()
    rigid_euler.SetMatrix(transform_martix.flatten())  # rotate rely on transducer axes axis
    rigid_euler.SetTranslation(move2target.GetOffset())  # move to target
    interpolator = sitk.sitkCosineWindowedSinc
    default_value = 0
    imageT = sitk.Resample(image, reference_image, rigid_euler, interpolator, default_value)

    return imageT



def make_skull_medium_rotation(filePath, width,  grid_spacing, transducer, target):

    image, simulation_grid = DICOM_read_rotation(filePath, width, grid_spacing, transducer, target)
    transform_matix = make_transform(transducer, target)
    crop_image = resample(target, image, simulation_grid, transform_matix)
    crop_image_arr = sitk.GetArrayFromImage(crop_image)
    crop_image_arr = crop_image_arr.transpose([2,1,0])

    # writer = sitk.ImageFileWriter()
    # writer.SetFileName("last_test.nii")
    # writer.Execute(crop_image)

    return crop_image_arr, crop_image


#sio.savemat('np_vector.mat', {'vect':crop_image_arr})



