import SimpleITK as sitk
import numpy as np
import scipy.io as sio

l2n = lambda l: np.array(l)
n2l = lambda n: list(n)



def DICOM_read_vertical(filePath, grid_spacing, tran_pos, target_pos):

    if filePath[-3:] ==".gz"  or filePath[-3:] =="nii":
        reader = sitk.ImageFileReader()
        reader.SetImageIO("NiftiImageIO")
        reader.SetFileName(filePath)
        image = reader.Execute()
    else:
        reader = sitk.ImageSeriesReader()
        dicom_names = reader.GetGDCMSeriesFileNames(filePath)
        reader.SetFileNames(dicom_names)
        image = reader.Execute()

    dist = np.linalg.norm(l2n(tran_pos) - l2n(target_pos))
    dirVec = (l2n(target_pos) - l2n(tran_pos)) / dist
    grid_max_point = target_pos + (dist * 1.15)
    grid_min_point = target_pos - (dist * 1.15)
    # grid_min_point[2] = grid_min_point[2]+dist*0.5

    grid_bound = np.abs(grid_max_point - grid_min_point)
    grid_number = np.round(grid_bound / grid_spacing)
    grid_number = grid_number - grid_number % 2

    reference_image = sitk.Image(int(grid_number[0]), int(grid_number[1]), int(grid_number[2]), sitk.sitkFloat32)
    reference_image.SetSpacing(grid_spacing)
    reference_image.SetOrigin(grid_min_point)
    reference_image[:, :, :] = 0

    return image, reference_image



def resample(image, reference_image):

    rigid_euler = sitk.Euler3DTransform()
    interpolator = sitk.sitkCosineWindowedSinc
    default_value = -1000.0
    imageT = sitk.Resample(image, reference_image, rigid_euler,interpolator, default_value)

    return imageT



def make_skull_medium_vertical(filePath, grid_spacing, transducer, target):

    image, simulation_grid = DICOM_read_vertical(filePath, grid_spacing, transducer, target)
    # transform_matix = make_transform(transducer, target)
    crop_image = resample(image, simulation_grid)
    crop_image_arr = sitk.GetArrayFromImage(crop_image)
    crop_image_arr = crop_image_arr.transpose([2,1,0])
    sio.savemat('ct.mat', {'ct': crop_image_arr})


    origin = crop_image.GetOrigin()
    dimension = crop_image_arr.shape

    x_end = origin[0] + grid_spacing[0] * (dimension[0] - 1)
    y_end = origin[1] + grid_spacing[1] * (dimension[1] - 1)
    z_end = origin[2] + grid_spacing[2] * (dimension[2] - 1)

    x_arr = np.linspace(origin[0], x_end, dimension[0])
    y_arr = np.linspace(origin[1], y_end, dimension[1])
    z_arr = np.linspace(origin[2], z_end, dimension[2])

    image_cordi = [x_arr, y_arr, z_arr]
    # writer = sitk.ImageFileWriter()
    # writer.SetFileName(savepath)
    # writer.Execute(crop_image)

    return crop_image_arr, image_cordi, crop_image


def make_skull_medium_vertical2(filePath, grid_spacing, focal_length, target):

    image, simulation_grid = DICOM_read_vertical2(filePath, grid_spacing, focal_length, target)
    # transform_matix = make_transform(transducer, target)
    crop_image = resample(image, simulation_grid)
    crop_image_arr = sitk.GetArrayFromImage(crop_image)
    #crop_image_arr = crop_image_arr.transpose([2,1,0])
    sio.savemat('ct.mat', {'ct': crop_image_arr})


    origin = crop_image.GetOrigin()
    dimension = crop_image_arr.shape

    x_end = origin[0] + grid_spacing[0] * (dimension[0] - 1)
    y_end = origin[1] + grid_spacing[1] * (dimension[1] - 1)
    z_end = origin[2] + grid_spacing[2] * (dimension[2] - 1)

    x_arr = np.linspace(origin[0], x_end, dimension[0])
    y_arr = np.linspace(origin[1], y_end, dimension[1])
    z_arr = np.linspace(origin[2], z_end, dimension[2])

    image_cordi = [x_arr, y_arr, z_arr]
    # writer = sitk.ImageFileWriter()
    # writer.SetFileName(savepath)
    # writer.Execute(crop_image)

    return crop_image_arr, image_cordi, crop_image

def DICOM_read_vertical2(filePath, grid_spacing, focal_length, target_pos):

    if filePath[-2:] == "gz":
        reader = sitk.ImageFileReader()
        reader.SetImageIO("NiftiImageIO")
        reader.SetFileName(filePath)

    elif filePath[-3:] == "nii":
        reader = sitk.ImageFileReader()
        reader.SetImageIO("NiftiImageIO")
        reader.SetFileName(filePath)

    elif filePath[-4:] == "nrrd":
        reader = sitk.ImageFileReader()
        reader.SetImageIO("NrrdImageIO")
        reader.SetFileName(filePath)
    else:
        reader = sitk.ImageSeriesReader()
        dicom_names = reader.GetGDCMSeriesFileNames(filePath)
        reader.SetFileNames(dicom_names)

    image = reader.Execute()
    grid_max_point = target_pos + (focal_length * 1.5)
    grid_min_point = target_pos - (focal_length * 1.5)
    # grid_min_point[2] = grid_min_point[2]+dist*0.5

    grid_bound = np.abs(grid_max_point - grid_min_point)
    grid_number = np.round(grid_bound / grid_spacing)
    grid_number = grid_number - grid_number % 2

    reference_image = sitk.Image(int(grid_number[0]), int(grid_number[1]), int(grid_number[2]), sitk.sitkFloat32)
    reference_image.SetSpacing(grid_spacing)
    reference_image.SetOrigin(grid_min_point)
#    reference_image[:, :, :] = -1000

    return image, reference_image

#sio.savemat('np_vector.mat', {'vect':crop_image_arr})



